#!/usr/bin/env python

# Config file
CONFIG_FILE = 'gitflow.config'

# Branches structure
DEFAULT_MASTER_BRANCH = 'master'
DEFAULT_STAGING_BRANCH = 'staging'

DEFAULT_HOTFIX_BRANCH = 'hotfix/{}'
DEFAULT_FEATURE_BRANCH = 'feature/{}'
DEFAULT_RELEASE_BRANCH = 'release/{}'

DEFAULT_VERSION = '{}-{}'

# Issue Classification - Api Labels
DEFAULT_FEATURE_LABELS = ['story', 'feature']
DEFAULT_CHANGE_LABELS = ['change']
DEFAULT_BUG_LABELS = ['bug']
DEFAULT_TECHNICAL_DEBT_LABELS = ['technical debt']
DEFAULT_IGNORE_LABELS = ['ignore']
DEFAULT_VERSION_LABEL_PREFIX = 'v.{major}.{minor}.{patch}'
